/*
 Project: Car Battery and charging system monitor
 MCU: PIC16F1827 running at 500KHz internal clock
 MCLR is disabled, and the battery voltage is
 measured through AN4 ADC channel

*/

// Define digit selection port pins
sbit DG1 at RA3_bit;
sbit DG2 at RA2_bit;
sbit DG3 at RA0_bit;
sbit DG4 at RA1_bit;
sbit DP1 at RB7_bit;



unsigned short i, DD1, DD2, DD3, DD4;
//unsigned int ADC_Value;
long DisplayVolt, ADC_Value;

//-------------- Function to Return mask for common cathode 7-seg. display
unsigned short mask(unsigned short num) {
  switch (num) {
    case 0 : return 0x3F;
    case 1 : return 0x06;
    case 2 : return 0x5B;
    case 3 : return 0x4F;
    case 4 : return 0x66;
    case 5 : return 0x6D;
    case 6 : return 0x7D;
    case 7 : return 0x07;
    case 8 : return 0x7F;
    case 9 : return 0x6F;
  } //case end
}

void Display_Data(){
  for (i = 0; i<=50; i++) {
      DG1 = 1;          // Select Ones Digit
      DG2 = 0;
      DG3 = 0;
      DG4 = 0;
      PORTB = DD1;
      delay_ms(5);
      DG1 = 0;          // Select Ones Digit
      DG2 = 1;
      DG3 = 0;
      DG4 = 0;
      PORTB = DD2;
      DP1 = 1;
      delay_ms(5);
      PORTB = 0;
      DG1 = 0;          // Select Ones Digit
      DG2 = 0;
      DG3 = 1;
      DG4 = 0;
      DP1 = 0;
      PORTB = DD3;
      delay_ms(5);
      PORTB = 0;
      DG1 = 0;          // Select Ones Digit
      DG2 = 0;
      DG3 = 0;
      DG4 = 1;
      PORTB = DD4;
      delay_ms(5);
      DG4 = 0;
      PORTB = 0;
   }
 }

unsigned Get_ADC(){
   ADCON0.F1 = 1;     // start conversion
   while (ADCON0.F1); // wait for conversion
   return (ADRESH << 8) + ADRESL; // return value
}

void main() {

 ANSELA = 0b00010000;  // RA4 analog input
 TRISA  = 0b00110000;  // RA4, RA5 inputs
 ANSELB = 0x00;        // PORTB all digital outputs
 TRISB  = 0x00;
 DD1 = mask(0);
 DD2 = mask(0);
 DD3 = mask(0);
 DD4 = mask(0);
 // Configure FVR to 4.096V
  FVRCON = 0b11000011 ;

  // Configure ADCON1
  ADCON1.ADNREF  = 0; // Vref- is connected to ground
  ADCON1.ADPREF0 = 1; // Vref+ is 4.096 V
  ADCON1.ADPREF1 = 1;
  ADCON1.ADCS0   = 0; // Use conversion clock, Fosc/2
  ADCON1.ADCS1   = 0; // Fosc = 500 KHz
  ADCON1.ADCS2   = 0;
  ADCON1.ADFM    = 1;  // result is right Justified

  // Configure ADCON0 for channel AN4
  ADCON0.CHS0 = 0;
  ADCON0.CHS1 = 0;
  ADCON0.CHS2 = 1;
  ADCON0.CHS3 = 0;
  ADCON0.CHS4 = 0;
  ADCON0.ADON = 1;  // enable A/D converter

  Display_Data();   // Display all zeros


 do {
  ADC_Value = Get_ADC();
  DisplayVolt = ADC_Value * 167;
  DD1 = DisplayVolt/100000;  // Extract DD1
  DD1 = mask(DD1);
  DD2 = (DisplayVolt/10000)%10;
  DD2 = mask(DD2);
  DD3 = (DisplayVolt/1000)%10;
  DD3 = mask(DD3);
  DD4 = (DisplayVolt/100)%10;
  DD4 = mask(DD4);
  Display_Data();
  Delay_ms(5);
 } while(1);


}